@@ -230,11 +230,11 @@ __________
 from abc import abstractmethod, ABC
 from typing import Any, Dict, List, Optional, Sequence
 
-from ae.core import DEBUG_LEVEL_VERBOSE                                             # type: ignore
-from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action       # type: ignore
+from ae.core import DEBUG_LEVEL_VERBOSE                                         # type: ignore
+from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.0.8'
+__version__ = '0.0.9'
 
 
 FLOW_PATH_ROOT_ID = '<<<root>>>'     #: needed for flow path jumper and for drop onto leave item button
@@ -313,6 +313,19 @@ class LiszDataMixin(ABC):
 
     # helper methods
 
+    def change_sub_node_sel(self, node: LiszNode, set_sel_to: bool):
+        """ change the selection of all the sub-leafs of the passed node to the specified value.
+
+        :param node:            node of which to change the selection of the sub-item leafs.
+        :param set_sel_to:      True will only toggle the unselected sub-item leafs, False only the selected ones.
+        """
+        for item in node:
+            item.pop('sel', None)       # always first remove sel left-overs
+            if 'node' in item:
+                self.change_sub_node_sel(item['node'], set_sel_to)
+            elif set_sel_to:
+                item['sel'] = 1.0
+
     def delete_item(self, item_id: str, node_only: bool):
         """ delete either the complete item or the sub node of the item (identified by the passed item id).
 
@@ -329,18 +342,12 @@ class LiszDataMixin(ABC):
 
         self.play_sound('deleted')
 
-    def edit_validate(self, old_idx: int,
-                      new_id: Optional[str] = None, want_node: Optional[bool] = None,
-                      toggle_sel: bool = False, toggle_unselected: bool = False) -> str:
+    def edit_validate(self, old_idx: int, new_id: Optional[str] = None, want_node: Optional[bool] = None) -> str:
         """ validate the user changes after adding a new item or editing an existing item.
 
         :param old_idx:             index in the current node of the edited item or -1 if a new item got edited.
         :param new_id:              new/edited id string.
         :param want_node:           True if the new/edited item will have a sub-node, False if not.
-        :param toggle_sel:          True if the selection state of the item has to be toggled.
-        :param toggle_unselected:   True will only toggle the unselected sub-item leafs, False only the selected ones.
-                                    This arg will only have an effect on toggling of a node item at the sub-leaf
-                                    selection states.
         :return:                    empty string on successful validation, else error string or
                                     `'request_delete_confirmation_for_item'` if the user has to confirm the deletion
                                     after the user wiped the item id string or
@@ -348,9 +355,7 @@ class LiszDataMixin(ABC):
                                     removal of the sub-node.
         """
         add_item = old_idx == -1
-        if new_id is None:
-            new_idx = old_idx
-        else:
+        if new_id is not None:
             if not new_id:
                 # on empty id cancel edit if add_item, else request confirmation from user for item del
                 return "" if add_item else 'request_delete_confirmation_for_item'
@@ -366,7 +371,6 @@ class LiszDataMixin(ABC):
             if want_node:
                 nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
             self.current_node_items.insert(0, nid)
-            new_idx = 0
 
         else:                               # edit item
             nid = self.current_node_items[old_idx]
@@ -380,9 +384,6 @@ class LiszDataMixin(ABC):
                 else:
                     nid.pop('node')         # remove empty node
 
-        if toggle_sel:
-            self.toggle_item_sel(new_idx, toggle_unselected)
-
         self.play_sound('added' if add_item else 'edited')
 
         return ""
@@ -581,16 +582,19 @@ class LiszDataMixin(ABC):
         """ initialize self.current_node_items from flow_id/flow_path saved on last app stop. """
         self.refresh_current_node_items_from_flow_path()
 
-    def on_filter_toggle(self, flow_id: str, _event_kwargs: Dict[str, Any]) -> bool:
+    def on_app_states_save(self, app_state):
+        """ shrink root_node app state variable before it get saved to the config file. """
+        self.shrink_node_size(app_state['root_node'])
+
+    def on_filter_toggle(self, toggle_attr: str, _event_kwargs: Dict[str, Any]) -> bool:
         """ toggle filter on click of either the selected or the unselected filter button.
 
         Note that the inverted filter may be toggled to prevent both filters active.
 
-        :param flow_id:         flow id, flow key is specifying the filter button.
+        :param toggle_attr:     specifying the filter button to toggle, either 'filter_selected' or 'filter_unselected'.
         :param _event_kwargs:   unused.
         :return:                True to process flow id change.
         """
-        toggle_attr = flow_key(flow_id)     # filter to toggle (either 'filter_selected' or 'filter_unselected')
         # inverted filter will be set to False if was True and toggled filter get changed to True.
         invert_attr = 'filter_unselected' if toggle_attr == 'filter_selected' else 'filter_selected'
 
@@ -661,36 +665,71 @@ class LiszDataMixin(ABC):
 
         return True         # key press processed
 
-    def on_item_enter(self, _flow_id: str, _event_kwargs: dict) -> bool:
+    def on_item_enter(self, _key: str, _event_kwargs: dict) -> bool:
         """ entering sub node from current node.
 
-        :param _flow_id:        flow id.
+        :param _key:            flow key (item id).
         :param _event_kwargs:   event kwargs.
         :return:                True for to process/change flow id.
         """
         self.play_sound(id_of_flow('enter', 'item'))
         return True
 
-    def on_item_leave(self, _flow_id: str, _event_kwargs: dict) -> bool:
+    def on_item_leave(self, _key: str, _event_kwargs: dict) -> bool:
         """ leaving sub node, setting current node to parent node.
 
-        :param _flow_id:        flow id.
+        :param _key:            flow key (item id).
         :param _event_kwargs:   event kwargs.
         :return:                True for to process/change flow id.
         """
         self.play_sound(id_of_flow('leave', 'item'))
         return True
 
-    def on_node_jump(self, flow_id: str, event_kwargs: dict) -> bool:
+    def on_item_sel_toggle(self, item_id: str, event_kwargs: dict) -> bool:
+        """ toggle selection of leaf item.
+
+        :param item_id:         item id of the leaf to toggle selection for.
+        :param event_kwargs:    event kwargs.
+        :return:                True for to process/change flow id.
+        """
+        self.toggle_item_sel(self.find_item_index(item_id))
+        event_kwargs['flow_id'] = id_of_flow('focus', 'item', item_id)
+        return True
+
+    def on_item_sel_change(self, item_id: str, event_kwargs: dict) -> bool:
+        """ toggle, set or reset in the current node the selection of a leaf item or of the sub-leafs of a node item.
+
+        :param item_id:         item id of the leaf to toggle selection for.
+        :param event_kwargs:    event kwargs, containing a `set_sel_to` key with a boolean value, where
+                                True will select and False deselect the item (or the sub-items if the item is a
+                                non-empty sub-node).
+        :return:                True for to process/change flow id.
+
+        This flow change event can be used alternatively to :meth:`~LiszDataMixin.on_item_sel_toggle`
+        for more sophisticated Lisz app implementations, like e.g. the
+        `kivy lisz demo app <https://gitlab.com/ae-group/kivy_lisz>`__ .
+        """
+        node_idx = self.find_item_index(item_id)
+        set_sel_to = event_kwargs['set_sel_to']
+        node = self.current_node_items[node_idx].get('node', list())
+        if node:
+            self.change_sub_node_sel(node, set_sel_to)
+        else:
+            assert not set_sel_to == bool(self.current_node_items[node_idx].get('sel', 0))
+            self.toggle_item_sel(node_idx)
+        event_kwargs['flow_id'] = id_of_flow('focus', 'item', item_id)
+        return True
+
+    def on_node_jump(self, flow_path_text: str, event_kwargs: dict) -> bool:
         """ FlowButton clicked event handler restoring flow path from the flow key.
 
         Used for to jump to node specified by the flow path text in the passed flow_id key.
 
-        :param flow_id:         jump node flow with the flow key containing the flow path text.
+        :param flow_path_text:  flow path text (identifying where to jump to).
         :param event_kwargs:    event arguments (not used here).
         :return:                True for to process/change flow.
         """
-        flow_path = self.flow_path_from_text(flow_key(flow_id))
+        flow_path = self.flow_path_from_text(flow_path_text)
 
         # cannot close popup here because the close event will be processed in the next event loop
         # an because flow_path_from_text() is overwriting the open popup action in self.flow_path
@@ -738,6 +777,16 @@ class LiszDataMixin(ABC):
         """ refresh current node including the depending display node. """
         self.current_node_items = self.flow_path_node()
 
+    def shrink_node_size(self, node: LiszNode):
+        """ shrink size of node and sub-node keys by removing unneeded `sel` keys.
+
+        :param node:            start or root node to shrink.
+        """
+        for item in node:
+            if 'node' in item:
+                item.pop('sel', None)
+                self.shrink_node_size(item['node'])
+
     def sub_item_ids(self, item_id, node_only, node=None, sub_item_ids=None):
         """ return item names of all items including their sub-node items (if exists).
 
@@ -766,21 +815,13 @@ class LiszDataMixin(ABC):
 
         return sub_item_ids
 
-    def toggle_item_sel(self, node_idx: int, toggle_unselected: bool = False):
-        """ update the item selection data of the item identified by the list index in the current node.
-
-        .. note::
-            Added `_ = toggle_unselected` to prevent pylint/PyCharm warnings, because this implementation does not use
-            the :paramref:`~toggle_item_sel.toggle_unselected` argument. Only some implementations (e.g.
-            <https://gitlab.com/ae-group/kivy_lisz>`_) allow to de-/select the sub-item leafs of a node
-            using this argument.
+    def toggle_item_sel(self, node_idx: int):
+        """ toggle the item selection of the item identified by the list index in the current node.
 
         :param node_idx:            list index of the item in the current node to change the selection for.
-        :param toggle_unselected:   True will only toggle the unselected sub-item leafs, False only the selected ones.
         """
-        _ = toggle_unselected
         selected = self.current_node_items[node_idx].get('sel', False)
         if selected:
             self.current_node_items[node_idx].pop('sel')
         else:
-            self.current_node_items[node_idx]['sel'] = 1
+            self.current_node_items[node_idx]['sel'] = 1.0
